@@ -84,7 +84,7 @@ use constant {
     simplefilenamere => qr/[-_\.0-9a-zA-Z]+/,
     pathre => qr/[- \p{Graph}_\.+!\$%&()\]\[;0-9a-zA-Z\/{}>*'@]+/,
     simplepathre => qr/[-_\$\.0-9a-zA-Z\/]+/,
-    commentre => qr/["-_\.+!\$%&()\]\[;0-9a-zA-Z\/ ]*/,
+    commentre => qr/[-"_\.+!\$%&()\]\[;\/'’ A-Za-z0-9ß\x{a1}-\x{ac}\x{ae}-\x{ff}\p{L}\p{Nd}\p{Zs}]*/,
     numberre => qr/[0-9]+/,
     namere => qr/^([^-+~:,\s\/][^:,\s\/]*)$/aa,
     anynamere => qr/^([^-+~:,\s\/][^:,\s\/]*)$/aa,
@@ -125,20 +125,42 @@ use constant {
 );
 
 sub sanitize_string {
-    my ($input, $pattern) = @_;
+    my ($input, $pattern, $replace) = @_;
+    return unless defined $input;
 
-    # Set a default pattern to allow alphanumeric characters,
-    # spaces, and underscores.
-    $pattern //= qr/[a-zA-Z0-9 _]*/;
-    log_trace("sanitize_string %s against %s", $input, $pattern);
+    $pattern //= '[a-zA-Z0-9 _]+';
+    $replace //= 0;
 
-    # If the input matches the pattern,
-    # extract and return the untainted value.
-    if ($input =~ qr/^($pattern)$/ ) {
-        log_trace("sanitize_string returning %s", "$1");
-        return $1;  # $1 is the captured, untainted portion of the string.
-    } else {
-        die "invalid characters in $input";
+    log_trace("sanitize_string %s against %s (replace=%s)", $input, $pattern, $replace);
+
+    # Full match check
+    if ($input =~ /^($pattern)\z/) {
+        log_trace("sanitize_string returning %s", $1);
+        return $1;  # Untainted
+    }
+    elsif ($replace) {
+        # Replace disallowed substrings with underscores
+        my $safe = '';
+        my $pos = 0;
+
+        while ($input =~ /$pattern/g) {
+            my $start = $-[0];  # start of match
+            my $end   = $+[0];  # end of match
+            # fill in underscores for skipped portion
+            $safe .= '_' x ($start - $pos) if $start > $pos;
+            # append allowed portion
+            $safe .= substr($input, $start, $end - $start);
+            $pos = $end;
+        }
+
+        # any trailing disallowed chars
+        $safe .= '_' x (length($input) - $pos) if $pos < length($input);
+
+        log_trace("sanitize_string replaced invalid chars, returning %s", $safe);
+        return $safe;
+    }
+    else {
+        die sprintf("sanitize_string: invalid characters in '%s'", $input);
     }
 }
 
